#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['doxybook', 'doxybook.templates_python']

package_data = \
{'': ['*'], 'doxybook': ['templates/*', 'templates/c/*', 'templates/cpp/*']}

install_requires = \
['Jinja2']

extras_require = \
{'doc': ['mkdocs', 'mkdocs-material']}

entry_points = \
{'console_scripts': ['esp-doxybook = doxybook.__main__:main']}

setup(name='esp-doxybook',
      version='0.1.0',
      description="Convert Doxygen XML to a single markdown file for C/C++'",
      author=None,
      author_email='Fu Hanxi <fuhanxi@espressif.com>',
      url=None,
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      entry_points=entry_points,
      python_requires='>=3.7',
     )
